//Sample debuggable JavaScript app
//Playing with colors with the Mozilla canvas object
//
//Loosely based on the tutorial
//"Making annoying rainbows in javascript"
//by Jim Bumgardner
//at http://www.krazydad.com/makecolors.php

var canvas = null;

function RGB2Color(r,g,b)   {
    return ['rgb(', r, ',', g, ',', b, ',0)'].join('');
}

function Context(canvas) {
    this.canvas = canvas;
    this.ctx = canvas.getContext("2d");
    if (!this.ctx) {
        throw new Exception("Couldn't get a context object");
    }
        
    // Defaults;
    this.height = canvas.height;
    this.width = 10;
    this.request_stop = false;
}

Context.prototype.stop = function()   {
    this.request_stop = true;
};

Context.prototype.RGB2Color = function(r,g,b)   {
    return ['rgb(', Math.round(r), ',', Math.round(g), ',', Math.round(b), ')'].join('');
}

Context.prototype.drawPoint = function(x,y,r,g,b) {
    var x1 = this.RGB2Color(r,g,b)
    this.ctx.fillStyle = x1;
    this.ctx.fillRect(x, y, this.width, this.height);
}

Context.prototype.draw = function() {
    this.frequency = .3;
    var x = 0;
    this.cwidth = this.canvas.getAttribute("width");
    var cheight = this.canvas.getAttribute("height");    
    this.ctx.clearRect(0, 0, this.cwidth, cheight);
    this.ypoint = (cheight - this.height) / 2;
    var i = 0;
    this.start_draw(i, x);
}

Context.prototype.start_draw = function(i, x) {
    var freq_i = this.frequency * i;
    var r = Math.sin(freq_i + 0) * 127 + 128;
    var g = Math.sin(freq_i + 2) * 127 + 128;
    var b  = Math.sin(freq_i + 4) * 127 + 128;
    this.drawPoint(x, this.ypoint, r, g, b);
    x += this.width;
    if (!this.request_stop && x < this.cwidth) {
        var self = this;
        setTimeout(function() {
            self.start_draw(i+1, x);
        }, 1000);
    } else {
        this.request_stop = false;
        //dump("All finished: x=" + x + ", cw = " + this.cwidth + "\n");
    }
}

function get_canvas() {
    return canvas || (canvas = new Context(document.getElementById('canvas')));
}

function draw() {
    var ctxt = get_canvas();
    this.request_stop = false;
    try {
        ctxt.draw();
    } catch(ex) {
        dump(ex + "\n");
    }
}

function stop() {
    if (canvas) {
        canvas.stop();
    }
}
