/*
 * Copyright (c) 2006, 2007 ActiveState Software Inc.
 * Refer to the license in the Komodo IDE installation
 * for the licensing information of this source code.
 *
 */

if (typeof(JSDBGP) == 'undefined') {
    var JSDBGP = {};
}
if (typeof(JSDBGP.Logging) == 'undefined') {
    JSDBGP.Logging = {};
}

JSDBGP.Logging._logs = {};
JSDBGP.Logging.getLogger = function(name) {
    if (typeof(JSDBGP.Logging._logs[name]) == 'undefined') {
        JSDBGP.Logging._logs[name] = new JSDBGP.Logging.Logger(name);
    }
    return JSDBGP.Logging._logs[name];
}
JSDBGP.Logging.OFF = -1;
JSDBGP.Logging.EXCEPTION = 0;
JSDBGP.Logging.ERROR = 1;
JSDBGP.Logging.WARN = 2;
JSDBGP.Logging.INFO = 3;
JSDBGP.Logging.DEBUG = 4;


    // XXX copied from venkman-utils.js
JSDBGP.Logging.getObjectTree = function(o, recurse, compress, level)
{
    var s = "";
    var pfx = "";

    if (typeof recurse == "undefined")
        recurse = 0;
    if (typeof level == "undefined")
        level = 0;
    if (typeof compress == "undefined")
        compress = true;

    var i;
    for (i = 0; i < level; i++)
        pfx += (compress) ? "| " : "|  ";

    var tee = (compress) ? "+ " : "+- ";

    if (typeof(o) != 'object') {
        s += pfx + tee + i + " (" + typeof(o) + ") " + o + "\n";
    } else
    for (i in o)
    {
        var t;
        try
        {
            t = typeof o[i];

            switch (t)
            {
                case "function":
                    var sfunc = String(o[i]).split("\n");
                    if (sfunc[2] == "    [native code]")
                        sfunc = "[native code]";
                    else
                        sfunc = sfunc.length + " lines";
                    s += pfx + tee + i + " (function) " + sfunc + "\n";
                    break;

                case "object":
                    s += pfx + tee + i + " (object) " + o[i] + "\n";
                    if (!compress)
                        s += pfx + "|\n";
                    if ((i != "parent") && (recurse))
                        s += JSDBGP.Logging.getObjectTree (o[i], recurse - 1,
                                             compress, level + 1);
                    break;

                case "string":
                    if (o[i].length > 1000)
                        s += pfx + tee + i + " (" + t + ") " +
                            o[i].length + " chars\n";
                    else
                        s += pfx + tee + i + " (" + t + ") '" + o[i] + "'\n";
                    break;

                default:
                    s += pfx + tee + i + " (" + t + ") " + o[i] + "\n";
            }
        }
        catch (ex)
        {
            s += pfx + tee + i + " (exception) " + ex + "\n";
        }

        if (!compress)
            s += pfx + "|\n";

    }

    s += pfx + "*\n";

    return s;

}

JSDBGP.Logging.Logger = function(name) {
    this.name = name;
}
JSDBGP.Logging.Logger.constructor = JSDBGP.Logging.Logger;
JSDBGP.Logging.Logger.prototype = {
    level: 2,
    lastErrorMsg: null,
    setLevel: function(level) {
        this.level = level;
    },
    doDump: function(level) {
        if (this.level >= level) {
            dump(this.name+": "+this.lastErrorMsg + "\n");
        }
    },
    debug: function(str) {
        this.lastErrorMsg = str;
        this.doDump(JSDBGP.Logging.DEBUG);
    },
    info: function(str) {
        this.lastErrorMsg = str;
        this.doDump(JSDBGP.Logging.INFO);
    },
    warn: function(str) {
        this.lastErrorMsg = str;
        this.doDump(JSDBGP.Logging.WARN);
    },
    error: function(str) {
        if (typeof(JSDBGP) == "undefined") {
            return;
        }
        this.lastErrorMsg = str;
        this.doDump(JSDBGP.Logging.ERROR);
    },
    exception: function(e, str) {
        if (typeof(JSDBGP) == "undefined") {
            return;
        }
        if (typeof(str) == 'undefined') {
            str = JSDBGP.Logging.getObjectTree(e, true);
        } else {
            str = str +"\n"+ JSDBGP.Logging.getObjectTree(e, true);
        }
        this.lastErrorMsg = str;
        this.doDump(JSDBGP.Logging.EXCEPTION);
    }
}
