/* -*- Mode: JavaScript; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (c) 2006, 2007 ActiveState Software Inc.
 * Refer to the license in the Komodo IDE installation
 * for the licensing information of this source code.
 *
 * winger-session.js -- manage the session with Komodo
 */

if (typeof(JSDBGP) == 'undefined') {
    var JSDBGP = {};
}

JSDBGP.Session = function(command_processor, viewObj, host_name, host_port) {
    this.command_processor = command_processor;
    this.host_name = host_name;
    this.host_port = host_port;
    this.log = JSDBGP.Logging.getLogger("winger.session");
    this.log.setLevel(JSDBGP.Logging.DEBUG);





    
    // XXX do we really need to connect on session creation?
    this.pending_input = "";
    this.input_pump_count = 0;
    this.connect(viewObj);
    this.running_id = 0;
    this.active_pump_ids = {};
    // Take up to 5 seconds to initialize the connection, else error out
    // Try every 500 milliseconds.  Net latency will make it more like
    // 1 second.
    this._retryDelay = 500; // msec
    this._connectionMaxTries = Math.ceil(5 * 1000 / this._retryDelay);
};

JSDBGP.Session.prototype.connect = function(viewObj) {
  try {
        var transportService =
            Components.classes["@mozilla.org/network/socket-transport-service;1"].getService(Components.interfaces.nsISocketTransportService);
        this.transport = transportService.createTransport(null, 0, this.host_name, this.host_port, null);
        if (!this.transport) {
            viewObj.showDisconnected();
            this.command_processor.error_code = JSDBGP.Constants.E_UNKNOWN;
            throw new Error("Unable to connect to "+this.host_name+":"+this.host_port)
        }

        this.raw_istream = this.transport.openInputStream(0, 0, 0);
        this.ostream = this.transport.openOutputStream(0,0,0);
        this.istream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
        this.istream.setInputStream(this.raw_istream);

        // Call the transport's isAlive function in a setTimeout, as it will not
        // work correctly until the javascript call stack has completely unwound.
        this._connectionTryCount = 0;   // Used to poll initial connected state
        window.setTimeout(function (me, viewObj) { me.finishConnecting(viewObj); }, 1, this, viewObj);
    } catch(ex) {



        alert("Komodo JavaScript Debugger: "+ex);
        this.disconnect();
        throw ex;
    }
};

JSDBGP.Session.prototype.finishConnecting = function(viewObj) {
    try {
        // Take up to 5 seconds to initialize the connection, else we error out
        // Example: Firewalled machine will fail after checking 5 times.
        JSDBGP.ui_set_status(JSDBGP.Constants.DBGP_STATE_CONNECTING);
        if (!this.transport.isAlive()) {
            if (this._connectionTryCount < this._connectionMaxTries) {
                this._connectionTryCount += 1;
                window.setTimeout(function (me, viewObj) { me.finishConnecting(viewObj); }, this._retryDelay, this, viewObj);
                return;
            } else {
                viewObj.showDisconnected();
                this.command_processor.error_code = JSDBGP.Constants.E_UNKNOWN;
                throw new Error("Connection is not alive for "+this.host_name+":"+this.host_port);
            }
        }
    
        // Don't create the input-pump until we enter a nestedEventLoop
        // this.make_input_pump();
    
        viewObj.showConnected();
  } catch(ex) {



      alert("Komodo JavaScript Debugger: "+ex);
      viewObj.showDisconnected();
      this.disconnect();
      throw ex;
  }
}

JSDBGP.Session.prototype.make_input_pump = function() {



    try {
        var pump = Components.classes["@mozilla.org/network/input-stream-pump;1"].createInstance(Components.interfaces.nsIInputStreamPump);
        pump.init(this.raw_istream, -1, -1, 0, 0, false);
        var this_id = ++this.running_id;
        pump.asyncRead(new JSDBGP.Session.StreamListener(this, this_id),
                       null);

    } catch (ex) {
        this.log.exception(ex);
        throw(ex);
    }
};

JSDBGP.Session.prototype.disconnect = function() {
    if ("istream" in this && this.istream)
        this.istream.close();
    if ("ostream" in this && this.ostream)
        this.ostream.close();
    if ("transport" in this && this.transport)
      this.transport.close(JSDBGP.Constants.NS_OK);

    this.istream = null;
    this.ostream = null;
    this.transport = null;
};

JSDBGP.Session.prototype.notifyLeavingNestedEventLoop = function() {
    if (this.input_pump_count > 0) {
        this.input_pump_count -= 1;
    }



};

//XXX - Pay attention to commands like 'break', 'stop', others?

// nsIBinaryInputStream doesn't support the readCString or readString()
// methods over sockets (https://bugzilla.mozilla.org/show_bug.cgi?id=331995),
// but reading a fixed number of bytes each time and checking for a null
// works.

// We can't read more than one byte at a time, because of how
// nsBinaryInputStream::ReadBytes is structured:
// rv = Read(s, aLength, &bytesRead);
// if (bytesRead != aLength)
//   return NS_ERROR_FAILURE;

// The DBGP protocol doesn't state how many bytes to read, just that it's
// null-terminated.  So we have to read one byte at a time.

// Keep the function in case something goes wrong.

JSDBGP.Session.prototype.get_input = function(count) {
    return this.istream.readBytes(count);
};

JSDBGP.Session.prototype.end_session = function() {
    try {



        this.input_pump_count = 0;
        this.disconnect();
    } catch(ex) {
        this.log.debug(ex, "end_session failed: ");
    }
};

JSDBGP.Session.prototype.writeData = function(data, dataLen) {
    try {
        if (!this.transport || !this.transport.isAlive()) {
            return;
        }
        var data2, dataLen2;
        if (arguments.length == 0) {



            return;
        } else if (arguments.length == 1) {
            dataLen2 = data.length;
            data2 = data;
        } else {
            dataLen2 = dataLen;
            data2 = data.substr(0, dataLen2);
        }

        var nullByte = String.fromCharCode(0);
        var dataLenStr = "" + dataLen2;
        var data3 = dataLenStr + nullByte + data2 + nullByte;
        var dataLen3 = dataLenStr.length + dataLen2 + 2; // 2 null bytes
        var str1 = this.expand(data3);
        if (str1.length > 1000) {
            str1 = str1.substr(0, 1000) + "...";
        }



        var num_written = this.ostream.write(data3, dataLen3);
        if (num_written != dataLen3) {






            if (num_written == 0) {



                this.ostream.close();
                this.command_processor.error_code = JSDBGP.Constants.E_UNKNOWN;
                throw new Error("Can't write to " + host + ":" + port);
            }
        }
    } catch(ex) {
        this.log.exception(ex, "writeData failed: ");
    }
}

JSDBGP.Session.prototype.expand = function(s) {
    // JS doesn't have foo ||= val
    if (!this._hexEscape) {
        this._hexEscape = function(str) {
            var res1 = parseInt(str.charCodeAt(0)).toString(16);
            var leader = res1.length == 1 ? "0" : "";
            return "%" + leader + res1;
        };
    }
    return s.replace(/[\x00-\x09\x11-\x1f]/g, this._hexEscape);
};

JSDBGP.Session.prototype.no_pending_pumps = function() {
    return this.keysLength(this.active_pump_ids) == 0;
};

JSDBGP.Session.prototype.keysLength = function(obj) {
    var rval = 0;
    for (var pump_id in this.active_pump_ids) {
        rval += 1;
    }
    return rval;
};

JSDBGP.Session.StreamListener = function(session, id) {
    this.session = session;
    this.id = id;
};

JSDBGP.Session.StreamListener.prototype.onStartRequest = function(request, context) {
    this.session.input_pump_count += 1;
    this.session.active_pump_ids[this.id] = null;






}

JSDBGP.Session.StreamListener.prototype.onStopRequest = function(request, context, status) {






    delete this.session.active_pump_ids[this.id];
    if (this.session.input_pump_count == 0
        && this.session.no_pending_pumps()) {



        this.session.disconnect();
    }







}

JSDBGP.Session.StreamListener.prototype.onDataAvailable = function(request, context, inputStream, offset, count) {
    var session = this.session;








    try {
        var buf = session.get_input(count);
        session.pending_input += buf;
        var nullByte = String.fromCharCode(0);
        while (session.pending_input.length > 0) {
            var end_pt = session.pending_input.indexOf(nullByte);
            if (end_pt < 0) {
                session.log.debug("Holding pending input of " + session.expand(session.pending_input));
                break;
            }
            if (end_pt > 0) {
                var cmd = session.pending_input.substr(0, end_pt);



                session.command_processor.dispatch(cmd);
            }
            session.pending_input = session.pending_input.substr(end_pt + 1);
        }
    } catch(ex) {
        session.log.exception("Failed to read " + count + " chars: " + ex);
        session.command_processor.fallback(ex);
    }
};

