/* -*- Mode: javascript; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (c) 2006, 2007 ActiveState Software Inc.
 * Refer to the license in the Komodo IDE installation
 * for the licensing information of this source code.
 *
 * winger-settings.js -- debugger settings
 */

if (typeof(JSDBGP) == 'undefined') {
  var JSDBGP = {};
}


JSDBGP.Settings = function() {
    this.log = JSDBGP.Logging.getLogger("winger.settings");
    this.log.setLevel(JSDBGP.Logging.DEBUG);
    this.init();

    this.registeredFunctions = {};
    
    this.registerFunction = function(name, owner, getterFunc, setterFunc) {
        this.registeredFunctions[name] = {obj : owner,
                                          getter: getterFunc,
                                          setter: setterFunc};
    };

    this.deregisterFunction = function(name) {
        delete this.registeredFunctions[name];
    };

    this.get = function(name) {
        var rf = this.registeredFunctions[name];
        return rf.getter.call(rf.obj);
    };

    this.set = function(name, featureValue) {
        var rf = this.registeredFunctions[name];
        return rf.setter.call(rf.obj, featureValue);
    };


    this.completeSettings = function(jsds) {
        if (typeof(jsds) == "undefined" || !jsds) {



            jsds = {'implementationMajor' : 1,
                    'implementationMinor' : 0};
        }
        this.settings.language_version = [jsds.implementationMajor + "." + jsds.implementationMinor];
    };
};


JSDBGP.Settings.prototype.init = function() {
    this.supportedCommands = {
        'status' : 1,
        'feature_get' : 1,
        'feature_set' : 1,
        'run' : 1,
        'step_into' : 1,
        'step_over' : 1,
        'step_out' : 1,
        'stop' : 1,
        'detach' : 1,
        'breakpoint_set' : 1,
        'breakpoint_get' : 1,
        'breakpoint_update' : 1,
        'breakpoint_remove' : 1,
        'breakpoint_list' : 1,
        'stack_depth' : 1,
        'stack_get' : 1,
        'context_names' : 1,
        'context_get' : 1,
        'typemap_get' : 1,
        'property_get' : 1,
        'property_set' : 1,
        'property_value' : 1,
        'source' : 1,
        'stdout' : 1,
        'stderr' : 1,
        'stdin' : 0,
        'break' : 1,
        'eval' : 1,
        'interact' : 1
    };
    
    // Feature name : [bool(3): is supported, is settable, has associated value]
    this.supportedFeatures = {
        'encoding' : [1, 1, 1],
        'data_encoding' : [1, 1, 1],
        'max_children' : [1, 1, 1],
        'max_data' : [1, 1, 1],
        'max_depth' : [1, 1, 1],
        'multiple_sessions' : [0, 0, 0],
        'notify_ok' : [1, 0, 1],
        'language_supports_threads' : [0, 0, 0],
        'language_name' : [1, 0, 1],
        'language_version' : [1, 0, 1],
        'protocol_version' : [1, 0, 1],
        'supports_async' : [1, 1, 1],
        'breakpoint_languages' : [1, 0, 1],
        'breakpoint_types' : [1, 0, 1],
        'breakpoint_threads' : [1, 0, 1],
        'pretty_print' : [1, 1, 1],
        'show_hidden' : [1, 1, 1],
        'urimap' : [1, 0]   // Values are handled by registered functions

        // Internal settings (that should be exposed to the IDE somehow)
        // None so far
    };
    
    // Feature name : [value, allowed settable values, if constrained]

    this.settings = {
        'encoding' : ['UTF-8', ['UTF-8', 'iso-8859-1']],
        'data_encoding' : ['base64', ['urlescape', 'base64', 'none', 'binary']],
        // binary  and 'none' are the same
        'max_children' : [10, 1],
        'max_data' : [32767, 1],
        'max_depth' : [1, 1],
        'language_name' : ['JavaScript'],
        'notify_ok' : [1, [0, 1]],
        
        'protocol_version' : ['1.0'],
        'supports_async' : [1, [0, 1]],
        
        'sort_ignore_at_signs' : [1, [0, 1]],
        'breakpoint_languages' : ['javascript,xml,html,xhtml'],
        'breakpoint_types' : ['line'],
        'breakpoint_threads' : [0],
        'pretty_print' : [0, [0,1]],
        'show_hidden' : [0, [0,1]]
                                  
    };
};


